

module trans_trend
!-----------------------------------------------------------------------
!
!      Quantile-quantile Transformation with Local Data
!      ************************************************
!
!
!
!
!
! AUTHOR: Clayton V. Deutsch                             DATE: 1989-1999
! MOD: Clayton V. Deutsch                                DATE: Aug. 2005
! MOD: Ryan M. Barnett - converted a stripped down version of the above
! program to subroutines. Intended for the PPMT program back-transform,
! where the target distribution is a N(0,1) distribution. DATE: Nov. 2013
!-----------------------------------------------------------------------
    contains

     subroutine krig_var( localfl, icoll, wtfac, kv )
        ! Read in the kriging variances and scale based on the 
        ! wtfac
        implicit none
        character*512, intent(in) :: localfl
        integer, intent(in) :: icoll(:)
        real*8, intent(in) :: wtfac
        real*8, intent(out) :: kv(:,:)
        integer lout, nvari, i, j, lkv, nvar, test, nxyz
        real*8, allocatable :: var(:)
        
        nvar = ubound( icoll, 1 )
        nxyz = ubound( kv, 1 )
        allocate( var(200) , stat = test)
        if(test /= 0)then; write(*,'(/,a,/)') 'ERROR: ALLOCATION FAILED';stop;endif
        open( lkv, file=localfl, err=95, status='unknown')
        read( lkv, *, err=95)
        read( lkv, *, err=95) nvari
        do i = 1, nvari
            read(lkv,*,err=95)
        end do
        do i = 1, nxyz
            read( lkv, *, err=95) var(1:nvari) 
            kv(i,:) = var(icoll(1:nvar))
        end do    
        close(lkv)
        do i = 1, nvar
            kv(:,i)= ( kv(:,i) / maxval(kv(:,i)) )**wtfac    
        enddo
        deallocate(var)
        return
        
95      write(*,*) 'Error with kriging variances file!'
        stop
      end subroutine krig_var

      subroutine ref_cdf( npass, tcdf )    
        ! Setup the N(0,1) reference distribution
        implicit none
        integer, intent(in) :: npass
        real*8, intent(out) :: tcdf(npass)
        integer ::  i
        real*8 :: oldcp, cp, sumwt
        tcdf  = 1.0 / real(npass)
        oldcp  = 0.0
        cp     = 0.0
        do i = 1, npass
            cp  = cp + tcdf(i)
            tcdf(i) = (cp + oldcp)  * 0.5d0
            oldcp   = cp
        end do
      end subroutine ref_cdf           
    
      subroutine trans( nloop, zin, tin, zval, kv )
          ! Perform the histogram matching transform
          use quicksort
          implicit none
          integer, intent(in) :: nloop
          real*8, intent(in) :: zin(:),tin(:)
          real*8, intent(in), optional :: kv(:)
          real*8, intent(out) :: zval(:)  
          real*8, allocatable, dimension(:) :: ix(:), tval(:)
          integer i, j, nxyz, test
          
          nxyz = ubound( zin, 1 )
          allocate( ix(nxyz), tval(nxyz) , stat = test)
          if(test /= 0)then; write(*,'(/,a,/)') 'ERROR: ALLOCATION FAILED';stop;endif
          
          tval = tin
          zval = zin
          do i = 1,nxyz
            ix(i) = i
          enddo
          !Now, loop to achieve data/distribution reproduction if local considerations:
          if( present(kv) )then
              ! Consider the local distributions
              call qsortem( zval, 1, nxyz, ix, kv )
              do i = 1, nloop
                    do j=1,nxyz
                          ! Modify by kriging variance if local data being accounted for:
                          zval(j) = zval(j) + kv(j)*( tval(j) - zval(j) )
                    end do
              end do
          else
             ! Simple transform without local considerations
             call qsortem( zval, 1, nxyz, ix )   
             zval = tval    
          endif
          ! Return to the original index
          zval(ix) = zval
      end subroutine trans
      
    end module trans_trend
